#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "SDL.h"

/* screen width, height, and bit depth */
#define SCREEN_WIDTH  640
#define SCREEN_HEIGHT 480
#define SCREEN_BPP     16

/* Set up some booleans */
#define TRUE  1
#define FALSE 0

/* Set up PI */
#define PI 3.14159265

GLfloat zoom ;
GLfloat pulsesin;
GLfloat xpos; 
GLfloat ypos; 
GLfloat xtux;
GLfloat ytux;

float scrollx;
char scrolltext[4096];
int scrollchar;
int scrollrotate;

float fastsin[7200];
float fastcos[7200];

int xmove; 
int ymove; 
int yvect;
int timecount;
int timenow;
int pulse;


#define SPARKS 100
GLfloat sparkx[SPARKS];
GLfloat sparky[SPARKS];
GLfloat sparkz[SPARKS];

/* This is our SDL surface */
SDL_Surface *surface;


GLuint texture[4]; /* Storage for some textures  */

/* function to release/destroy our resources and restoring the old desktop */
void Quit( int returnCode )
{
    /* clean up the window */
    SDL_Quit( );

    /* and exit appropriately */
    exit( returnCode );
}

/* function to load in bitmap as a GL texture */
int LoadGLTextures( )
{
	int loop;
	
        SDL_Surface *TextureImage[4]; 
	TextureImage[0] = SDL_LoadBMP( "data/cd.bmp"); 
	TextureImage[1] = SDL_LoadBMP( "data/tux.bmp"); 
	TextureImage[2] = SDL_LoadBMP( "data/spark.bmp"); 
	TextureImage[3] = SDL_LoadBMP( "data/font.bmp"); 
	glGenTextures( 4, &texture[0] );

        for (loop=0; loop<4; loop++)
	{	
		glBindTexture( GL_TEXTURE_2D, texture[loop] );
		glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage[loop]->w,
		TextureImage[loop]->h, 0, GL_BGR,
		GL_UNSIGNED_BYTE, TextureImage[loop]->pixels );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
                if ( TextureImage[loop] )
                	SDL_FreeSurface( TextureImage[loop] );
      	}

    return TRUE;
}

/* function to reset our viewport after a window resize */
int resizeWindow( int width, int height )
{
    /* Height / width ration */
    GLfloat ratio;
 
    /* Protect against a divide by zero */
    if ( height == 0 )
	height = 1;

    ratio = ( GLfloat )width / ( GLfloat )height;

    /* Setup our viewport. */
    glViewport( 0, 0, ( GLint )width, ( GLint )height );

    /*
     * change to the projection matrix and set
     * our viewing volume.
     */
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );

    /* Set our perspective */
    gluPerspective( 30.0f, ratio, 0.1f, 100.0f );

    /* Make sure we're chaning the model view and not the projection */
    glMatrixMode( GL_MODELVIEW );

    /* Reset The View */
    glLoadIdentity( );

    return( TRUE );
}

/* function to handle key press events */
void handleKeyPress( SDL_keysym *keysym )
{
    switch ( keysym->sym )
	{
	case SDLK_ESCAPE:
	    /* ESC key was pressed */
	    Quit( 0 );
	    break;
	case SDLK_F1:
	    /* F1 key was pressed
	     * this toggles fullscreen mode
	     */
	    SDL_WM_ToggleFullScreen( surface );
	    break;
	default:
	    break;
	}

    return;
}

/* general OpenGL initialization function */

void init_fastmath(void)
{
	int t;
 for (t=0; t< 7200; t++)	
	{	
	fastsin[t]=(sin(t * PI / 360 / 10 ));
	fastcos[t]=(cos(t * PI / 360 / 10 ));
	}
}

void init_starfield(void)
{
	int t;
 	for (t=0; t< SPARKS; t++)	
	{	
	sparkx[t]=(rand() %10) * 0.4 - 1;
	sparky[t]=(rand() %10) * 0.3 - 2;
	sparkz[t]=(rand() %10) * 0.5 - 0.05;
	}
}
void GLfontchar ( char textin[], float xin, float yin, float zin1, float zin2)
	{
	float fontmapx;
	float fontmapy;
	fontmapx =0.625; fontmapy =0.625;
        
                if ( textin[0] == 'A' ) { fontmapx =0.000; fontmapy =0.000; }
                if ( textin[0] == 'B' ) { fontmapx =0.125; fontmapy =0.000; }
                if ( textin[0] == 'C' ) { fontmapx =0.250; fontmapy =0.000; }
                if ( textin[0] == 'D' ) { fontmapx =0.375; fontmapy =0.000; }
                if ( textin[0] == 'E' ) { fontmapx =0.500; fontmapy =0.000; }
                if ( textin[0] == 'F' ) { fontmapx =0.625; fontmapy =0.000; }
                if ( textin[0] == 'G' ) { fontmapx =0.750; fontmapy =0.000; }
                if ( textin[0] == 'H' ) { fontmapx =0.875; fontmapy =0.000; }

                if ( textin[0] == 'I' ) { fontmapx =0.000; fontmapy =0.125; }
                if ( textin[0] == 'J' ) { fontmapx =0.125; fontmapy =0.125; }
                if ( textin[0] == 'K' ) { fontmapx =0.250; fontmapy =0.125; }
                if ( textin[0] == 'L' ) { fontmapx =0.375; fontmapy =0.125; }
                if ( textin[0] == 'M' ) { fontmapx =0.500; fontmapy =0.125; }
                if ( textin[0] == 'N' ) { fontmapx =0.625; fontmapy =0.125; }
                if ( textin[0] == 'O' ) { fontmapx =0.750; fontmapy =0.125; }
                if ( textin[0] == 'P' ) { fontmapx =0.875; fontmapy =0.125; }

                if ( textin[0] == 'Q' ) { fontmapx =0.000; fontmapy =0.250; }
                if ( textin[0] == 'R' ) { fontmapx =0.125; fontmapy =0.250; }
                if ( textin[0] == 'S' ) { fontmapx =0.250; fontmapy =0.250; }
                if ( textin[0] == 'T' ) { fontmapx =0.375; fontmapy =0.250; }
                if ( textin[0] == 'U' ) { fontmapx =0.500; fontmapy =0.250; }
                if ( textin[0] == 'V' ) { fontmapx =0.625; fontmapy =0.250; }
                if ( textin[0] == 'W' ) { fontmapx =0.750; fontmapy =0.250; }
                if ( textin[0] == 'X' ) { fontmapx =0.875; fontmapy =0.250; }

                if ( textin[0] == 'Y' ) { fontmapx =0.000; fontmapy =0.375; }
                if ( textin[0] == 'Z' ) { fontmapx =0.125; fontmapy =0.375; }
                if ( textin[0] == '0' ) { fontmapx =0.250; fontmapy =0.375; }
                if ( textin[0] == '1' ) { fontmapx =0.375; fontmapy =0.375; }
                if ( textin[0] == '2' ) { fontmapx =0.500; fontmapy =0.375; }
                if ( textin[0] == '3' ) { fontmapx =0.625; fontmapy =0.375; }
                if ( textin[0] == '4' ) { fontmapx =0.750; fontmapy =0.375; }
                if ( textin[0] == '5' ) { fontmapx =0.875; fontmapy =0.375; }

                if ( textin[0] == '6' ) { fontmapx =0.000; fontmapy =0.500; }
                if ( textin[0] == '7' ) { fontmapx =0.125; fontmapy =0.500; }
                if ( textin[0] == '8' ) { fontmapx =0.250; fontmapy =0.500; }
                if ( textin[0] == '9' ) { fontmapx =0.375; fontmapy =0.500; }
                if ( textin[0] == '.' ) { fontmapx =0.500; fontmapy =0.500; }
                if ( textin[0] == '!' ) { fontmapx =0.750; fontmapy =0.500; }
                if ( textin[0] == '?' ) { fontmapx =0.875; fontmapy =0.500; }

                if ( textin[0] == '=' ) { fontmapx =0.000; fontmapy =0.625; }
                if ( textin[0] == '@' ) { fontmapx =0.125; fontmapy =0.625; }

      		glTexCoord2f( 0.0 + fontmapx, 0.12 + fontmapy );
      			glVertex3f( -0.2f + xin, -0.1 + yin, zin1 );
      		glTexCoord2f( 0.125 + fontmapx, 0.12 + fontmapy );
      			glVertex3f(  0.2f + xin, -0.1 +yin, zin1 );
      		glTexCoord2f( 0.125 + fontmapx, 0.0 + fontmapy );
      			glVertex3f(  0.2f + xin,  0.1 + yin, zin2 );
      		glTexCoord2f( 0.0 + fontmapx, 0.0 + fontmapy );
      			glVertex3f( -0.2f + xin, 0.1 + yin, zin2 );
	}

int initGL( GLvoid )
{

    /* Load in the texture */
    if ( !LoadGLTextures( ) )
	return FALSE;

    /* Enable Texture Mapping ( NEW ) */
    glEnable( GL_TEXTURE_2D );

    /* Enable smooth shading */
    glShadeModel( GL_FLAT );

    glEnable(GL_BLEND);
 
    /* Set the background black */
    glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );

    /* Depth buffer setup */
    glClearDepth( 1.0f );

    /* Enables Depth Testing */
    glEnable( GL_DEPTH_TEST );

    /* The Type Of Depth Test To Do */
    glDepthFunc( GL_LEQUAL );

    /* Really Nice Perspective Calculations */
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

    return( TRUE );
}

/* Here goes our drawing code */
int drawGLScene( GLvoid )
{
    int loop;

    /* Clear The Screen And The Depth Buffer */
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glLoadIdentity( );
    glTranslatef( 0.0f, 0.5f, -5.0f );


    /* doing scroller */

    glBindTexture( GL_TEXTURE_2D, texture[3] );
    glBlendFunc(GL_SRC_COLOR, GL_ONE);
    glColor4f(1.0f,1.0f,1.0f,1.0f);
    glBegin(GL_QUADS);
	for (loop=0; loop< 42; loop+=1)
      	{
		GLfontchar ( &scrolltext[loop + scrollchar],  -0.2f + loop * 0.4 - scrollx, -2.3, fastcos[scrollrotate] - 4 , fastsin[scrollrotate] - 5);
	}
  glEnd( );

    /* doing tux */

    glBindTexture( GL_TEXTURE_2D, texture[1] );
    glColor4f(1.0f,1.0f,1.0f,1.0f);
    glBlendFunc(GL_SRC_COLOR, GL_ONE);
    glBegin(GL_QUADS);
	for (loop=0; loop<7200; loop+=360)
      	{
	xtux = fastsin[loop]*1.8;
	ytux = sin((loop + yvect)  * PI / 3600 ) * 0.6;
		
      		glTexCoord2f( 0.0f, 1.0f ); glVertex3f( -0.2f + xtux, -0.2f + ytux, -1.0f );
      		glTexCoord2f( 1.0f, 1.0f ); glVertex3f(  0.2f + xtux, -0.2f + ytux, -1.0f );
      		glTexCoord2f( 1.0f, 0.0f ); glVertex3f(  0.2f + xtux,  0.2f + ytux, -1.0f );
      		glTexCoord2f( 0.0f, 0.0f ); glVertex3f( -0.2f + xtux,  0.2f + ytux, -1.0f );
	}
  glEnd( );

    /* doing the logo */
    glBindTexture( GL_TEXTURE_2D, texture[0] );
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glBegin(GL_QUADS);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
      	glTexCoord2f( 0.0f, 1.0f ); glVertex3f( -1.0f + xpos, -0.25f + ypos, 0.0f );
      	glTexCoord2f( 1.0f, 1.0f ); glVertex3f(  1.0f + xpos, -0.25f + ypos, 0.0f );
      	glTexCoord2f( 1.0f, 0.0f ); glVertex3f(  1.0f + xpos,  0.25f + ypos, 0.0f );
      	glTexCoord2f( 0.0f, 0.0f ); glVertex3f( -1.0f + xpos,  0.25f + ypos, 0.0f );
	for (loop=0; loop<15; loop++)
      	{
		zoom = loop * 0.03 ;
                glColor4f(1.0f,1.0f,1.0f, ((0.2 - loop * 0.01) - pulsesin ));
      		glTexCoord2f( 0.0f, 1.0f ); glVertex3f( -1.0f + xpos, -0.25f + ypos, zoom );
      		glTexCoord2f( 1.0f, 1.0f ); glVertex3f(  1.0f + xpos, -0.25f + ypos, zoom );
      		glTexCoord2f( 1.0f, 0.0f ); glVertex3f(  1.0f + xpos,  0.25f + ypos, zoom );
      		glTexCoord2f( 0.0f, 0.0f ); glVertex3f( -1.0f + xpos,  0.25f + ypos, zoom );
	}
  glEnd( );

    /* doing sparks */

    glBindTexture( GL_TEXTURE_2D, texture[2] );
    glColor4f(1.0f,1.0f,1.0f,0.5f);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glBegin(GL_TRIANGLES);
	for (loop=0; loop<SPARKS; loop++)
      	{
		sparkx[loop] -=0.01;
		
		if ( sparkx[loop] < -2.0)
		{
			 sparkx[loop]=2.0 + (rand() %10) * 0.2 ;
			 sparky[loop]=(rand() %10) * 0.3 - 2;
			 sparkz[loop]=(rand() %10) * 0.5 - 0.05;
		}
                glColor4f(1.0f,1.0f,1.0f, 1.0f);
      		glTexCoord2f( 0.0f, 0.0f ); glVertex3f( sparkx[loop], sparky[loop], sparkz[loop]);
      		glTexCoord2f( 0.0f, 1.0f ); glVertex3f( sparkx[loop], sparky[loop] - 0.01f, sparkz[loop]);
      		glTexCoord2f( 1.0f, 0.0f ); glVertex3f( sparkx[loop] + 0.05f, sparky[loop], sparkz[loop]);
	}
  glEnd( );



   /* Draw it to the screen */
    SDL_GL_SwapBuffers( );

    	xmove += 10;
    	ymove += 5;
    	pulse += 2;
	pulsesin = fastsin[pulse*10]*0.1;
	xpos = fastcos[xmove]*0.3;
	ypos = fastsin[xmove]*0.3;

	scrollx += 0.01;
	scrollrotate += 36;
	
	if (scrollx > 3.9)
		{
		scrollx = 3.5;
		scrollchar +=1;
		}

	if (scrollchar > strlen(scrolltext))
		scrollchar = 0;

	if (scrollrotate >= 7200)
		scrollrotate = 0;
			

	yvect +=10;

    	if (yvect >= 7200 )
    		yvect=0;

    	if (pulse >= 360 )
    		pulse=0;

    	if (xmove >= 7190 )
    		xmove = 0;

    	if (ymove >= 7190 )
    		ymove = 0;

    	return( TRUE );
}

int main( int argc, char **argv )
{

    /* Flags to pass to SDL_SetVideoMode */
    int videoFlags;


    /* main loop variable */
    int done = FALSE;
    /* used to collect events */
    SDL_Event event;
    /* this holds some info about our display */
    const SDL_VideoInfo *videoInfo;

    init_fastmath();
    srand(SDL_GetTicks());
    init_starfield();

    strcpy( scrolltext, "                      ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 .!?=@" );
    scrollx=3;

    /* initialize SDL */
    if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
	    fprintf( stderr, "Video initialization failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* Fetch the video info */
    videoInfo = SDL_GetVideoInfo( );

    if ( !videoInfo )
	{
	    fprintf( stderr, "Video query failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* the flags to pass to SDL_SetVideoMode */
    videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL */
    videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering */
    videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
    videoFlags |= SDL_RESIZABLE;       /* Enable window resizing */

    /* This checks to see if surfaces can be stored in memory */
    if ( videoInfo->hw_available )
	videoFlags |= SDL_HWSURFACE;
    else
	videoFlags |= SDL_SWSURFACE;

    /* This checks if hardware blits can be done */
    if ( videoInfo->blit_hw )
	videoFlags |= SDL_HWACCEL;

    /* Sets up OpenGL double buffering */
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

    /* get a SDL surface */
    surface = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,
				videoFlags );

    /* Verify there is a surface */
    if ( !surface )
	{
	    fprintf( stderr,  "Video mode set failed: %s\n", SDL_GetError( ) );
	    Quit( 1 );
	}

    /* initialize OpenGL */
    initGL( );

    SDL_ShowCursor(0);

    /* resize the initial window */
    resizeWindow( SCREEN_WIDTH, SCREEN_HEIGHT );

    /* wait for events */
    while ( !done )
	{
	    /* handle the events in the queue */

	    while ( SDL_PollEvent( &event ) )
		{
		    switch( event.type )
			{
			case SDL_VIDEORESIZE:
			    /* handle resize event */
			    surface = SDL_SetVideoMode( event.resize.w,
							event.resize.h,
							16, videoFlags );
			    if ( !surface )
				{
				    fprintf( stderr, "Could not get a surface after resize: %s\n", SDL_GetError( ) );
				    Quit( 1 );
				}
			    resizeWindow( event.resize.w, event.resize.h );
			    break;
			case SDL_KEYDOWN:
			    /* handle key presses */
			    handleKeyPress( &event.key.keysym );
			    break;
			case SDL_QUIT:
			    /* handle quit requests */
			    done = TRUE;
			    break;
			default:
			    break;
			}
		}

    		timenow = SDL_GetTicks();
    		if ( (timecount + 1) < timenow )
    			{
			timecount =  SDL_GetTicks() ;
			SDL_Delay(1) ;
		    	/* draw the scene */
			drawGLScene( );
		}
	}

    /* clean ourselves up and exit */
    Quit( 0 );

    /* Should never get here */
    return( 0 );
}
